import { App } from 'vue';
import { InjectionKey } from 'vue';

export declare class CheckboxInput extends InputBase<boolean> {
    type: string;
}

export declare function createDynamicForms(options?: DynamicFormsOptions): DynamicFormsPlugin;

export declare interface DynamicFormsOptions {
    autoValidate?: boolean;
    form?: FormOptions;
}

export declare interface DynamicFormsPlugin {
    options?: DynamicFormsOptions;
    install(app: App): void;
}

export declare const dynamicFormsSymbol: InjectionKey<DynamicFormsPlugin>;

export declare const email: (control: FormControl<any>) => {
    email: boolean;
};

export declare class EmailInput extends InputBase<string> {
    type: string;
}

export declare class FormControl<T> extends InputBase<T> {
    valid: boolean;
    invalid: boolean;
    dirty: boolean;
    touched: boolean;
    errors: ValidationErrors | null;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}

export declare interface FormOptions {
    customClass?: string;
    method?: string;
    netlify?: boolean;
    netlifyHoneypot?: string;
}

export declare class FormValidation {
    validator: ValidatorFn;
    text: string;
    constructor(validator: ValidatorFn, text: string);
}

export declare class InputBase<T> {
    value: T | undefined;
    name: string;
    label: string;
    disabled?: boolean;
    order?: number;
    type: string;
    customClass?: string;
    placeholder?: string;
    validations?: FormValidation[];
    options?: {
        key: string;
        value: string;
        disabled?: boolean;
    }[];
    cols?: number;
    rows?: number;
    constructor(options?: {
        value?: T | undefined;
        name?: string;
        label?: string;
        disabled?: boolean;
        order?: number;
        type?: string;
        customClass?: string;
        placeholder?: string;
        validations?: FormValidation[];
        options?: {
            key: string;
            value: string;
            disabled?: boolean;
        }[];
        cols?: number;
        rows?: number;
    });
}

export declare const isEmptyInputValue: (value: any) => boolean;

export declare const max: (max: number) => (control: FormControl<any>) => {
    max: {
        max: number;
        actual: any;
    };
};

export declare const maxLength: (maxLength: number) => (control: FormControl<any>) => {
    maxlength: {
        requiredLength: number;
        actualLength: any;
    };
};

export declare const min: (min: number) => (control: FormControl<any>) => {
    min: {
        min: number;
        actual: number;
    };
};

export declare const minLength: (minLength: number) => (control: FormControl<any>) => {
    minlength: {
        requiredLength: number;
        actualLength: any;
    };
};

export declare class PasswordInput extends InputBase<string> {
    type: string;
}

export declare const pattern: (pattern: string) => (control: any) => {
    pattern: {
        requiredPattern: string | RegExp;
        actualValue: any;
    };
};

export declare class RadioInput extends InputBase<boolean> {
    type: string;
}

export declare const required: (control: FormControl<any>) => {
    required: boolean;
};

export declare class SelectInput<T> extends InputBase<T> {
    type: string;
}

export declare class TextAreaInput extends InputBase<string> {
    type: string;
}

export declare class TextInput extends InputBase<string> {
    type: string;
}

export declare const url: (control: FormControl<any>) => {
    email: boolean;
};

export declare function useDynamicForms(): DynamicFormsPlugin;

declare type ValidationErrors = {
    [key: string]: any;
};

declare interface ValidatorFn {
    (control: FormControl<any> | undefined): ValidationErrors | null;
}

export { }
