/*!
 * @asigloo/vue-dynamic-forms v1.x.x
 * (c) 2020 Alvaro Saburido
 * @license MIT
 */
var asVueDynamicForms = (function (exports, vue) {
  'use strict';

  const dynamicFormsSymbol = Symbol();
  function useDynamicForms() {
      const dynamicForms = vue.inject(dynamicFormsSymbol);
      if (!dynamicForms)
          throw new Error('No dynamicForms provided!!!');
      return dynamicForms;
  }

  function useInputEvents(control, emit) {
      function onChange($event) {
          if (control) {
              control.value = $event.target.value;
              control.dirty = true;
          }
          emit('changed', $event.target.value);
      }
      function onCheck($event) {
          if (control) {
              control.value = $event.target.checked;
              control.dirty = true;
          }
          emit('changed', $event.target.checked);
      }
      function onFocus() {
          emit('focus');
      }
      function onBlur() {
          emit('blur');
          if (control) {
              control.touched = true;
          }
      }
      return {
          onFocus,
          onChange,
          onBlur,
          onCheck,
      };
  }

  const props = {
      control: Object,
  };
  var script = vue.defineComponent({
      name: 'asTextInput',
      props,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          return () => {
              var _a, _b, _c, _d, _e;
              return vue.h('input', {
                  name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                  type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                  disabled: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.disabled,
                  placeholder: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.placeholder,
                  class: ['form-control'],
                  value: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.value,
                  onFocus,
                  onBlur,
                  onChange,
              });
          };
      },
  });

  const render = () => {};


  script.render = render;
  script.__file = "src/components/text-input/TextInput.vue";

  const props$1 = {
      control: Object,
  };
  var script$1 = vue.defineComponent({
      name: 'asSelectInput',
      props: props$1,
      setup(props, { emit }) {
          return () => {
              var _a, _b, _c, _d, _e, _f;
              const { onChange, onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
              const options = (_b = (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map(({ key, value, disabled }) => vue.h('option', { key, value, disabled }, value));
              return vue.h('select', {
                  name: ((_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.name) || '',
                  disabled: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.disabled,
                  placeholder: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.placeholder,
                  class: ['form-control'],
                  value: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.value,
                  onFocus,
                  onBlur,
                  onChange,
              }, options);
          };
      },
  });

  const render$1 = () => {};


  script$1.render = render$1;
  script$1.__file = "src/components/select-input/SelectInput.vue";

  const props$2 = {
      control: Object,
  };
  var script$2 = vue.defineComponent({
      name: 'asTextAreaInput',
      props: props$2,
      setup(props, { emit }) {
          const { onChange, onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          return () => {
              var _a, _b, _c, _d, _e, _f;
              return vue.h('textarea', {
                  name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                  disabled: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.disabled,
                  placeholder: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.placeholder,
                  class: ['form-control'],
                  value: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.value,
                  rows: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.rows,
                  cols: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.cols,
                  onFocus,
                  onBlur,
                  onChange,
              });
          };
      },
  });

  const render$2 = () => {};


  script$2.render = render$2;
  script$2.__file = "src/components/text-area-input/TextAreaInput.vue";

  const props$3 = {
      control: Object,
  };
  var script$3 = vue.defineComponent({
      name: 'asCheckboxInput',
      props: props$3,
      setup(props, { emit }) {
          const { onCheck, onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          return () => {
              var _a, _b, _c, _d, _e, _f, _g;
              return [
                  vue.h('input', {
                      name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                      type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                      id: (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.name,
                      disabled: (_d = props === null || props === void 0 ? void 0 : props.control) === null || _d === void 0 ? void 0 : _d.disabled,
                      class: ['checkbox-control'],
                      value: (_e = props === null || props === void 0 ? void 0 : props.control) === null || _e === void 0 ? void 0 : _e.value,
                      onFocus,
                      onBlur,
                      onChange: onCheck,
                  }),
                  vue.h('label', {
                      class: ['checkbox-label'],
                      for: (_f = props === null || props === void 0 ? void 0 : props.control) === null || _f === void 0 ? void 0 : _f.name,
                  }, (_g = props === null || props === void 0 ? void 0 : props.control) === null || _g === void 0 ? void 0 : _g.label),
              ];
          };
      },
  });

  const render$3 = () => {};


  script$3.render = render$3;
  script$3.__file = "src/components/checkbox-input/CheckboxInput.vue";

  const props$4 = {
      control: Object,
  };
  var script$4 = vue.defineComponent({
      name: 'asRadioInput',
      props: props$4,
      setup(props, { emit }) {
          var _a, _b;
          const { onChange, onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          const renderRadios = (_b = (_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.options) === null || _b === void 0 ? void 0 : _b.map(option => {
              var _a, _b;
              return vue.h('div', { class: 'radio-input' }, [
                  vue.h('input', {
                      name: ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.name) || '',
                      type: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.type,
                      id: option.key,
                      disabled: option.disabled,
                      class: ['radio-control'],
                      value: option.value,
                      onFocus,
                      onBlur,
                      onChange,
                  }),
                  vue.h('label', {
                      class: ['radio-label'],
                      for: option.key,
                  }, option.value),
              ]);
          });
          return () => [
              vue.h('div', {
                  class: 'radio-group',
                  tabIndex: -1,
                  role: 'group',
              }, renderRadios),
          ];
      },
  });

  const render$4 = () => {};


  script$4.render = render$4;
  script$4.__file = "src/components/radio-input/RadioInput.vue";

  const { assign, entries, values, keys } = Object;
  const slugify = (text) => text
      .toString()
      .toLowerCase()
      .replace(/\s+/g, '-') // Replace spaces with -
      //eslint-disable-next-line
      .replace(/[^\w\-]+/g, '') // Remove all non-word chars
      .replace(/--+/g, '-') // Replace multiple - with single -
      .replace(/^-+/, '') // Trim - from start of text
      .replace(/-+$/, '');
  const isArray = (a) => !!a && a.constructor === Array;
  const isObject = (a) => !!a && a.constructor === Object;
  const isEmpty = (entry) => {
      if (isArray(entry)) {
          return entry.length > 0;
      }
      if (isObject(entry)) {
          return entries(entry).length > 0;
      }
  };

  /* eslint-disable @typescript-eslint/no-use-before-define */
  const components = {
      TextInput: script,
      SelectInput: script$1,
      TextAreaInput: script$2,
      CheckboxInput: script$3,
      RadioInput: script$4,
  };
  const props$5 = {
      control: {
          type: Object,
          required: true,
      },
      submited: {
          type: Boolean,
          required: true,
      },
  };
  var script$5 = vue.defineComponent({
      name: 'asDynamicInput',
      components,
      props: props$5,
      setup(props, { emit, slots }) {
          const { onFocus, onBlur } = useInputEvents(props === null || props === void 0 ? void 0 : props.control, emit);
          const { options } = vue.inject(dynamicFormsSymbol);
          let component;
          const attributes = vue.computed(() => {
              return {
                  control: props.control,
                  onChanged: valueChange,
              };
          });
          const hasLabel = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) !== 'checkbox'; });
          const isFieldSet = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === 'radio'; });
          const getClasses = vue.computed(() => {
              var _a, _b;
              return [
                  'dynamic-input',
                  'form-group',
                  {
                      'form-group--inline': ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) === 'checkbox',
                  },
                  {
                      'form-group--error': showErrors.value,
                  },
                  `${((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.customClass) || ''}`,
              ];
          });
          const autoValidate = vue.computed(() => { var _a; return ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.touched) && (options === null || options === void 0 ? void 0 : options.autoValidate); });
          const showErrors = vue.computed(() => {
              var _a, _b;
              return (((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.errors) &&
                  keys((_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.errors).length > 0 &&
                  (props.submited || autoValidate.value));
              /* props.control.errors &&
                Object.keys(props.control.errors).length > 0 &&
                (this.submited || this.autoValidate) */
          });
          const errorMessages = vue.computed(() => {
              var _a;
              const errors = values(((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.errors) || {});
              if (errors.length > 0 && (props.submited || autoValidate.value)) {
                  return errors.map(value => value.text);
              }
              return [];
          });
          function validate() {
              if (props.control &&
                  props.control.validations &&
                  isEmpty(props.control.validations)) {
                  const validation = props.control.validations.reduce((prev, curr) => {
                      const val = typeof curr.validator === 'function'
                          ? curr.validator(props.control)
                          : null;
                      if (val !== null) {
                          const [key, value] = entries(val)[0];
                          const obj = {};
                          obj[key] = {
                              value,
                              text: curr.text,
                          };
                          return {
                              ...prev,
                              ...obj,
                          };
                      }
                      return {
                          ...prev,
                      };
                  }, {});
                  props.control.errors = validation;
                  props.control.valid = Object.keys(validation).length === 0;
              }
          }
          function valueChange(value) {
              const newValue = {};
              if (props.control) {
                  newValue[props.control.name] = value;
                  validate();
                  emit('changed', newValue);
              }
          }
          return () => {
              var _a, _b, _c;
              switch ((_a = props === null || props === void 0 ? void 0 : props.control) === null || _a === void 0 ? void 0 : _a.type) {
                  case 'text':
                  case 'email':
                  case 'password':
                  case 'url':
                      component = vue.h(script, attributes.value);
                      break;
                  case 'select':
                      component = vue.h(script$1, attributes.value);
                      break;
                  case 'textarea':
                      component = vue.h(script$2, attributes.value);
                      break;
                  case 'checkbox':
                      component = vue.h(script$3, attributes.value);
                      break;
                  case 'radio':
                      component = vue.h(script$4, attributes.value);
                      break;
                  case 'custom-field':
                      component = vue.h('slot', {
                          name: 'customField',
                      }, slots.customField({
                          control: props.control,
                          onChange: ($event) => {
                              const newValue = {};
                              const value = $event.target.value;
                              if (props.control) {
                                  props.control.dirty = true;
                                  newValue[props.control.name] = value;
                                  validate();
                                  emit('changed', newValue);
                              }
                          },
                          onFocus,
                          onBlur,
                      }));
                      break;
              }
              return vue.h(isFieldSet.value ? 'fieldset' : 'div', {
                  class: getClasses.value,
                  role: isFieldSet.value ? undefined : 'group',
              }, [
                  hasLabel.value
                      ? vue.h(isFieldSet.value ? 'legend' : 'label', {
                          class: 'form-label',
                          for: (_b = props === null || props === void 0 ? void 0 : props.control) === null || _b === void 0 ? void 0 : _b.label,
                      }, (_c = props === null || props === void 0 ? void 0 : props.control) === null || _c === void 0 ? void 0 : _c.label)
                      : null,
                  component,
                  vue.h('div', {
                      class: 'form-errors',
                  }, errorMessages.value.map(error => vue.h('p', { class: 'form-error' }, error))),
              ]);
          };
      },
  });

  const render$5 = () => {};


  script$5.render = render$5;
  script$5.__file = "src/components/dynamic-input/DynamicInput.vue";

  class FormValidation {
      constructor(validator, text) {
          this.validator = validator;
          this.text = text || '';
      }
  }
  class InputBase {
      constructor(options = {}) {
          this.value = options.value;
          this.name = options.name || slugify(options.label || '') || '';
          this.label = options.label || '';
          this.disabled = !!options.disabled;
          this.order = options.order === undefined ? 1 : options.order;
          this.type = options.type || '';
          this.customClass = options.customClass;
          this.placeholder = options.placeholder;
          this.validations = options.validations || [];
          this.options = options.options;
          this.rows = options.rows || 0;
          this.cols = options.cols || 0;
      }
  }
  class TextInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'text';
      }
  }
  class EmailInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'email';
      }
  }
  class PasswordInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'password';
      }
  }
  class SelectInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'select';
      }
  }
  class TextAreaInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'textarea';
      }
  }
  class CheckboxInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'checkbox';
      }
  }
  class RadioInput extends InputBase {
      constructor() {
          super(...arguments);
          this.type = 'radio';
      }
  }
  class FormControl extends InputBase {
      constructor(options = {}) {
          super({
              value: options.value,
              name: options.name,
              label: options.label,
              type: options.type,
              disabled: !!options.disabled,
              order: options.order,
              customClass: options.customClass,
              validations: options.validations,
              options: options.options,
              rows: options.rows,
              cols: options.cols,
          });
          this.valid = true;
          this.invalid = false;
          this.dirty = false;
          this.touched = false;
          this.errors = {};
      }
  }

  /* import { warn } from '../../core/utils/warning';
   */
  const props$6 = {
      form: {
          type: Object,
          required: true,
      },
  };
  const components$1 = {
      DynamicInput: script$5,
  };
  /* const AVAILABLE_THEMES = ['default', 'material'];
   */
  var script$6 = vue.defineComponent({
      name: 'asDynamicForm',
      props: props$6,
      components: components$1,
      setup(props, ctx) {
          const { options } = vue.inject(dynamicFormsSymbol);
          const controls = vue.ref([]);
          const formValues = vue.reactive({});
          const submited = vue.ref(false);
          vue.onMounted(() => {
              mapControls();
              initValues();
          });
          //  TODO: enable again when plugin theme option is available
          /* const validTheme = computed(
            () => options.theme && AVAILABLE_THEMES.includes(options.theme),
          );
      
          if (!validTheme.value) {
            warn(
              `There isn't a theme: ${
                options.theme
              } just yet, please choose one of the available themes: ${AVAILABLE_THEMES.join(
                ', ',
              )}`,
            );
          } */
          const deNormalizedScopedSlots = vue.computed(() => Object.keys(ctx.slots));
          const normalizedControls = vue.computed(() => {
              let normalizedControls = {};
              controls.value.forEach(element => {
                  normalizedControls[element.name] = element;
              });
              return normalizedControls;
          });
          const isValid = vue.computed(() => {
              var _a;
              const control = (_a = controls === null || controls === void 0 ? void 0 : controls.value) === null || _a === void 0 ? void 0 : _a.find(control => !control.valid);
              return control ? control.valid : true;
          });
          const errors = vue.computed(() => {
              return controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const errors = Object.keys(curr.errors || {}) || [];
                      if (errors.length > 0) {
                          const error = {};
                          error[curr.name] = curr.errors;
                          return {
                              ...prev,
                              ...error,
                          };
                      }
                      return prev;
                  }, {})
                  : {};
          });
          const formattedOptions = vue.computed(() => {
              if (options === null || options === void 0 ? void 0 : options.form) {
                  const { customClass, method, netlify, netlifyHoneypot } = options === null || options === void 0 ? void 0 : options.form;
                  return {
                      class: [
                          customClass,
                      ].join(' '),
                      method,
                      'data-netlify': netlify,
                      'data-netlify-honeypot': netlifyHoneypot,
                  };
              }
              else {
                  return;
              }
          });
          function valueChange(changedValue) {
              Object.assign(formValues, changedValue);
              ctx.emit('changed', formValues);
          }
          function mapControls(empty) {
              var _a, _b;
              controls.value =
                  ((_b = (_a = props.form) === null || _a === void 0 ? void 0 : _a.fields) === null || _b === void 0 ? void 0 : _b.map((field) => empty
                      ? new FormControl({ ...field, value: null })
                      : new FormControl({ ...field }))) || [];
          }
          function resetForm() {
              mapControls(true);
          }
          function handleSubmit() {
              submited.value = true;
              if (isValid.value) {
                  ctx.emit('submited', formValues);
                  resetForm();
              }
              else {
                  ctx.emit('error', formValues);
              }
          }
          function initValues() {
              Object.assign(formValues, controls.value
                  ? controls.value.reduce((prev, curr) => {
                      const obj = {};
                      obj[curr.name] =
                          curr.type === 'number' ? parseFloat(curr.value) : curr.value;
                      return {
                          ...prev,
                          ...obj,
                      };
                  }, {})
                  : {});
              ctx.emit('changed', formValues);
          }
          vue.watch(props, () => {
              mapControls();
          });
          return {
              controls,
              form: props.form,
              valueChange,
              formValues,
              handleSubmit,
              isValid,
              errors,
              deNormalizedScopedSlots,
              normalizedControls,
              submited,
              formattedOptions,
          };
      },
  });

  function render$6(_ctx, _cache, $props, $setup, $data, $options) {
    const _component_dynamic_input = vue.resolveComponent("dynamic-input");

    return (vue.openBlock(), vue.createBlock("form", vue.mergeProps({
      class: "dynamic-form",
      novalidate: "",
      id: _ctx.form.id,
      name: _ctx.form.id
    }, _ctx.formattedOptions, {
      onSubmit: _cache[1] || (_cache[1] = vue.withModifiers((...args) => (_ctx.handleSubmit(...args)), ["prevent"]))
    }), [
      (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.controls, (control) => {
        return (vue.openBlock(), vue.createBlock(_component_dynamic_input, {
          key: control.name,
          control: control,
          submited: _ctx.submited,
          onChanged: _ctx.valueChange
        }, {
          customField: vue.withCtx((props) => [
            (vue.openBlock(true), vue.createBlock(vue.Fragment, null, vue.renderList(_ctx.deNormalizedScopedSlots, (slot) => {
              return (vue.openBlock(), vue.createBlock("div", {
                key: slot,
                class: "custom-form-wrapper"
              }, [
                (props.control.name === slot)
                  ? vue.renderSlot(_ctx.$slots, slot, {
                      key: 0,
                      control: _ctx.normalizedControls[slot],
                      onChange: props.onChange,
                      onFocus: props.onFocus,
                      onBlur: props.onBlur
                    })
                  : vue.createCommentVNode("v-if", true)
              ]))
            }), 128 /* KEYED_FRAGMENT */))
          ]),
          _: 2
        }, 1032 /* PROPS, DYNAMIC_SLOTS */, ["control", "submited", "onChanged"]))
      }), 128 /* KEYED_FRAGMENT */))
    ], 16 /* FULL_PROPS */, ["id", "name"]))
  }

  script$6.render = render$6;
  script$6.__file = "src/components/dynamic-form/DynamicForm.vue";

  function createDynamicForms(options) {
      const vdf = {
          options,
          install(app) {
              const self = this;
              app.component('dynamic-form', script$6);
              app.provide(dynamicFormsSymbol, self);
          },
      };
      return vdf;
  }

  const isEmptyInputValue = (value) => value == null || value === '' || value.length === 0;
  const EMAIL_REGEXP = /^(?=.{1,254}$)(?=.{1,64}@)[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+(\.[-!#$%&'*+/0-9=?A-Z^_`a-z{|}~]+)*@[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?(\.[A-Za-z0-9]([A-Za-z0-9-]{0,61}[A-Za-z0-9])?)*$/;
  const URL_REGEXP = /^((?:(https?):\/\/)?((?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9])\.)(?:(?:25[0-5]|2[0-4][0-9]|1[0-9][0-9]|[0-9][0-9]|[0-9]))|(?:(?:(?:\w+\.){1,2}[\w]{2,3})))(?::(\d+))?((?:\/[\w]+)*)(?:\/|(\/[\w]+\.[\w]{3,4})|(\?(?:([\w]+=[\w]+)&)*([\w]+=[\w]+))?|\?(?:(wsdl|wadl))))$/;
  const required = (control) => isEmptyInputValue(control.value) ? { required: true } : null;
  const min = (min) => (control) => {
      if (isEmptyInputValue(control.value) || isEmptyInputValue(min)) {
          return null; // don't validate empty values to allow optional controls
      }
      const value = parseFloat(control.value);
      return !isNaN(value) && value < min
          ? { min: { min, actual: +control.value } }
          : null;
  };
  const max = (max) => (control) => {
      if (isEmptyInputValue(control.value) || isEmptyInputValue(max)) {
          return null; // don't validate empty values to allow optional controls
      }
      const value = parseFloat(control.value);
      // Controls with NaN values after parsing should be treated as not having a
      // maximum, per the HTML forms spec: https://www.w3.org/TR/html5/forms.html#attr-input-max
      return !isNaN(value) && value > max
          ? { max: { max, actual: control.value } }
          : null;
  };
  const email = (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      return EMAIL_REGEXP.test(control.value) ? null : { email: true };
  };
  const url = (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      return URL_REGEXP.test(control.value) ? null : { email: true };
  };
  const minLength = (minLength) => (control) => {
      if (isEmptyInputValue(control.value)) {
          return null; // don't validate empty values to allow optional controls
      }
      const length = control.value ? control.value.length : 0;
      return length < minLength
          ? { minlength: { requiredLength: minLength, actualLength: length } }
          : null;
  };
  const maxLength = (maxLength) => (control) => {
      const length = control.value ? control.value.length : 0;
      return length > maxLength
          ? { maxlength: { requiredLength: maxLength, actualLength: length } }
          : null;
  };
  const pattern = (pattern) => {
      if (!pattern)
          return null;
      let regex;
      let regexStr;
      if (typeof pattern === 'string') {
          regexStr = '';
          if (pattern.charAt(0) !== '^')
              regexStr += '^';
          regexStr += pattern;
          if (pattern.charAt(pattern.length - 1) !== '$')
              regexStr += '$';
          regex = new RegExp(regexStr);
      }
      return control => {
          if (isEmptyInputValue(control.value)) {
              return null; // don't validate empty values to allow optional controls
          }
          const value = control.value;
          return regex.test(value)
              ? null
              : { pattern: { requiredPattern: regexStr, actualValue: value } };
      };
  };

  exports.CheckboxInput = CheckboxInput;
  exports.EmailInput = EmailInput;
  exports.FormControl = FormControl;
  exports.FormValidation = FormValidation;
  exports.InputBase = InputBase;
  exports.PasswordInput = PasswordInput;
  exports.RadioInput = RadioInput;
  exports.SelectInput = SelectInput;
  exports.TextAreaInput = TextAreaInput;
  exports.TextInput = TextInput;
  exports.createDynamicForms = createDynamicForms;
  exports.dynamicFormsSymbol = dynamicFormsSymbol;
  exports.email = email;
  exports.isEmptyInputValue = isEmptyInputValue;
  exports.max = max;
  exports.maxLength = maxLength;
  exports.min = min;
  exports.minLength = minLength;
  exports.pattern = pattern;
  exports.required = required;
  exports.url = url;
  exports.useDynamicForms = useDynamicForms;

  return exports;

}({}, Vue));
